@@ -47,7 +47,7 @@ class Command:
         self.requires_wallet = 'w' in s
         self.requires_password = 'p' in s
         self.description = func.__doc__
-        self.help = self.description.split('.')[0]
+        self.help = self.description.split('.')[0] if self.description else None
         varnames = func.func_code.co_varnames[1:func.func_code.co_argcount]
         self.defaults = func.func_defaults
         if self.defaults:
@@ -66,8 +66,8 @@ def command(s):
         name = func.__name__
         known_commands[name] = Command(func, s)
         @wraps(func)
-        def func_wrapper(*args):
-            return func(*args)
+        def func_wrapper(*args, **kwargs):
+            return func(*args, **kwargs)
         return func_wrapper
     return decorator
 
@@ -94,26 +94,33 @@ class Commands:
         return result
 
     @command('')
-    def help(self):
-        """Print help"""
-        return 'Commands: ' + ', '.join(sorted(known_commands.keys()))
+    def commands(self):
+        """List of commands"""
+        return ' '.join(sorted(known_commands.keys()))
 
     @command('')
     def create(self):
         """Create a new wallet"""
+        raise BaseException('Not a JSON-RPC command')
 
-    @command('')
-    def restore(self):
-        """Restore a wallet from seed. """
+    @command('wn')
+    def restore(self, text):
+        """Restore a wallet from text. Text can be a seed phrase, a master
+        public key, a master private key, a list of bitcoin addresses
+        or bitcoin private keys. If you want to be prompted for your
+        seed, type '?' or ':' (concealed) """
+        raise BaseException('Not a JSON-RPC command')
 
-    @command('')
+    @command('w')
     def deseed(self):
         """Remove seed from wallet. This creates a seedless, watching-only
         wallet."""
+        raise BaseException('Not a JSON-RPC command')
 
     @command('wp')
     def password(self):
         """Change wallet password. """
+        raise BaseException('Not a JSON-RPC command')
 
     @command('')
     def getconfig(self, key):
@@ -148,14 +155,16 @@ class Commands:
         """Return the transaction history of any address. Note: This is a
         walletless server query, results are not checked by SPV.
         """
-        return self.network.synchronous_get([('blockchain.address.get_history', [address])])[0]
+        return self.network.synchronous_get(('blockchain.address.get_history', [address]))
 
     @command('nw')
     def listunspent(self):
         """List unspent outputs. Returns the list of unspent transaction
         outputs in your wallet."""
         l = copy.deepcopy(self.wallet.get_spendable_coins(exclude_frozen = False))
-        for i in l: i["value"] = str(Decimal(i["value"])/COIN)
+        for i in l:
+            v = i["value"]
+            i["value"] = float(v)/COIN if v is not None else None
         return l
 
     @command('n')
@@ -163,16 +172,15 @@ class Commands:
         """Returns the UTXO list of any address. Note: This
         is a walletless server query, results are not checked by SPV.
         """
-        return self.network.synchronous_get([('blockchain.address.listunspent', [address])])[0]
+        return self.network.synchronous_get(('blockchain.address.listunspent', [address]))
 
     @command('n')
     def getutxoaddress(self, txid, pos):
         """Get the address of a UTXO. Note: This is a walletless server query, results are
         not checked by SPV.
         """
-        r = self.network.synchronous_get([('blockchain.utxo.get_address', [txid, pos])])
-        if r:
-            return {'address':r[0]}
+        r = self.network.synchronous_get(('blockchain.utxo.get_address', [txid, pos]))
+        return {'address': r}
 
     @command('wp')
     def createrawtx(self, inputs, outputs, unsigned=False):
@@ -193,7 +201,7 @@ class Commands:
         tx = Transaction.from_io(tx_inputs, outputs)
         if not unsigned:
             self.wallet.sign_transaction(tx, self.password)
-        return tx
+        return tx.as_dict()
 
     @command('wp')
     def signtransaction(self, tx, privkey=None):
@@ -205,7 +213,7 @@ class Commands:
             t.sign({pubkey:privkey})
         else:
             self.wallet.sign_transaction(t, self.password)
-        return t
+        return t.as_dict()
 
     @command('')
     def deserialize(self, tx):
@@ -217,7 +225,7 @@ class Commands:
     def broadcast(self, tx):
         """Broadcast a transaction to the network. """
         t = Transaction(tx)
-        return self.network.synchronous_get([('blockchain.transaction.broadcast', [str(t)])])[0]
+        return self.network.synchronous_get(('blockchain.transaction.broadcast', [str(t)]))
 
     @command('')
     def createmultisig(self, num, pubkeys):
@@ -239,20 +247,21 @@ class Commands:
 
     @command('wp')
     def getprivatekeys(self, address):
-        """Get the private keys of an address. Address must be in wallet."""
-        return self.wallet.get_private_key(address, self.password)
+        """Get private keys of addresses. You may pass a single wallet address, or a list of wallet addresses."""
+        is_list = type(address) is list
+        domain = address if is_list else [address]
+        out = [self.wallet.get_private_key(address, self.password) for address in domain]
+        return out if is_list else out[0]
 
     @command('w')
     def ismine(self, address):
         """Check if address is in wallet. Return true if and only address is in wallet"""
         return self.wallet.is_mine(address)
 
-    @command('wp')
-    def dumpprivkeys(self, domain=None):
-        """Dump private keys from your wallet"""
-        if domain is None:
-            domain = self.wallet.addresses(True)
-        return [self.wallet.get_private_key(address, self.password) for address in domain]
+    @command('')
+    def dumpprivkeys(self):
+        """Deprecated."""
+        return "This command is deprecated. Use a pipe instead: 'electrum listaddresses | electrum getprivatekeys - '"
 
     @command('')
     def validateaddress(self, address):
@@ -284,7 +293,7 @@ class Commands:
         """Return the balance of any address. Note: This is a walletless
         server query, results are not checked by SPV.
         """
-        out = self.network.synchronous_get([('blockchain.address.get_balance', [address])])[0]
+        out = self.network.synchronous_get(('blockchain.address.get_balance', [address]))
         out["confirmed"] =  str(Decimal(out["confirmed"])/COIN)
         out["unconfirmed"] =  str(Decimal(out["unconfirmed"])/COIN)
         return out
@@ -292,7 +301,7 @@ class Commands:
     @command('n')
     def getproof(self, address):
         """Get Merkle branch of an address in the UTXO set"""
-        p = self.network.synchronous_get([('blockchain.address.get_proof', [address])])[0]
+        p = self.network.synchronous_get(('blockchain.address.get_proof', [address]))
         out = []
         for i,s in p:
             out.append(i)
@@ -302,7 +311,7 @@ class Commands:
     def getmerkle(self, txid, height):
         """Get Merkle branch of a transaction included in a block. Electrum
         uses this to verify transactions (Simple Payment Verification)."""
-        return self.network.synchronous_get([('blockchain.transaction.get_merkle', [txid, int(height)])])[0]
+        return self.network.synchronous_get(('blockchain.transaction.get_merkle', [txid, int(height)]))
 
     @command('n')
     def getservers(self):
@@ -322,6 +331,11 @@ class Commands:
         """Get master public key. Return your wallet\'s master public key(s)"""
         return self.wallet.get_master_public_keys()
 
+    @command('wp')
+    def getmasterprivate(self):
+        """Get master private key. Return your wallet\'s master private key"""
+        return str(self.wallet.get_master_private_key(self.wallet.root_name, self.password))
+
     @command('wp')
     def getseed(self):
         """Get seed phrase. Print the generation seed of your wallet."""
@@ -348,15 +362,16 @@ class Commands:
 
     @command('n')
     def sweep(self, privkey, destination, tx_fee=None, nocheck=False):
-        """Sweep private key. Returns a transaction that spends UTXOs from
+        """Sweep private keys. Returns a transaction that spends UTXOs from
         privkey to a destination address. The transaction is not
         broadcasted."""
+        privkeys = privkey if type(privkey) is list else [privkey]
         self.nocheck = nocheck
         dest = self._resolver(destination)
         if tx_fee is None:
             tx_fee = 0.0001
         fee = int(Decimal(tx_fee)*COIN)
-        return Transaction.sweep([privkey], self.network, dest, fee)
+        return Transaction.sweep(privkeys, self.network, dest, fee)
 
     @command('wp')
     def signmessage(self, address, message):
@@ -389,53 +404,33 @@ class Commands:
                         self.wallet.add_input_info(i)
                     output = ('address', address, amount)
                     dummy_tx = Transaction.from_io(inputs, [output])
-                    fee = self.wallet.estimated_fee(dummy_tx)
+                    fee_per_kb = self.wallet.fee_per_kb(self.config)
+                    fee = self.wallet.estimated_fee(dummy_tx, fee_per_kb)
                 amount -= fee
             else:
                 amount = int(COIN*Decimal(amount))
             final_outputs.append(('address', address, amount))
 
         coins = self.wallet.get_spendable_coins(domain)
-        tx = self.wallet.make_unsigned_transaction(coins, final_outputs, fee, change_addr)
+        tx = self.wallet.make_unsigned_transaction(coins, final_outputs, self.config, fee, change_addr)
         str(tx) #this serializes
         if not unsigned:
             self.wallet.sign_transaction(tx, self.password)
         return tx
 
-    def _read_csv(self, csvpath):
-        import csv
-        outputs = []
-        with open(csvpath, 'rb') as csvfile:
-            csvReader = csv.reader(csvfile, delimiter=',')
-            for row in csvReader:
-                address, amount = row
-                assert bitcoin.is_address(address)
-                amount = Decimal(amount)
-                outputs.append((address, amount))
-        return outputs
-
-    @command('wp')
-    def payto(self, destination, amount, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, deserialized=False, broadcast=False):
+    @command('wpn')
+    def payto(self, destination, amount, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False):
         """Create a transaction. """
         domain = [from_addr] if from_addr else None
         tx = self._mktx([(destination, amount)], tx_fee, change_addr, domain, nocheck, unsigned)
-        if broadcast:
-            r, h = self.wallet.sendtx(tx)
-            return h
-        else:
-            return tx.deserialize() if deserialized else tx
+        return tx.as_dict()
 
-    @command('wp')
-    def paytomany(self, csv_file, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False, deserialized=False, broadcast=False):
+    @command('wpn')
+    def paytomany(self, outputs, tx_fee=None, from_addr=None, change_addr=None, nocheck=False, unsigned=False):
         """Create a multi-output transaction. """
         domain = [from_addr] if from_addr else None
-        outputs = self._read_csv(csv_file)
         tx = self._mktx(outputs, tx_fee, change_addr, domain, nocheck, unsigned)
-        if broadcast:
-            r, h = self.wallet.sendtx(tx)
-            return h
-        else:
-            return tx.deserialize() if deserialized else tx
+        return tx.as_dict()
 
     @command('wn')
     def history(self):
@@ -445,13 +440,18 @@ class Commands:
         for item in self.wallet.get_history():
             tx_hash, conf, value, timestamp, balance = item
             try:
-                time_str = datetime.datetime.fromtimestamp( timestamp).isoformat(' ')[:-3]
+                time_str = datetime.datetime.fromtimestamp(timestamp).isoformat(' ')[:-3]
             except Exception:
                 time_str = "----"
-
             label, is_default_label = self.wallet.get_label(tx_hash)
-
-            out.append({'txid':tx_hash, 'date':"%16s"%time_str, 'label':label, 'value':format_satoshis(value), 'confirmations':conf})
+            out.append({
+                'txid':tx_hash,
+                'timestamp':timestamp,
+                'date':"%16s"%time_str,
+                'label':label,
+                'value':float(value)/COIN if value is not None else None,
+                'confirmations':conf}
+            )
         return out
 
     @command('w')
@@ -481,7 +481,7 @@ class Commands:
 
     @command('w')
     def listaddresses(self, receiving=False, change=False, show_labels=False, frozen=False, unused=False, funded=False, show_balance=False):
-        """List wallet addresses. Returns your list of addresses."""
+        """List wallet addresses. Returns the list of all addresses in your wallet. Use optional arguments to filter the results."""
         out = []
         for addr in self.wallet.addresses(True):
             if frozen and not self.wallet.is_frozen(addr):
@@ -503,16 +503,16 @@ class Commands:
         return out
 
     @command('nw')
-    def gettransaction(self, txid, deserialized=False):
+    def gettransaction(self, txid):
         """Retrieve a transaction. """
         tx = self.wallet.transactions.get(txid) if self.wallet else None
         if tx is None and self.network:
-            raw = self.network.synchronous_get([('blockchain.transaction.get', [txid])])[0]
+            raw = self.network.synchronous_get(('blockchain.transaction.get', [txid]))
             if raw:
                 tx = Transaction(raw)
             else:
                 raise BaseException("Unknown transaction")
-        return tx.deserialize() if deserialized else tx
+        return tx.as_dict()
 
     @command('')
     def encrypt(self, pubkey, message):
@@ -531,7 +531,7 @@ class Commands:
             PR_PAID: 'Paid',
             PR_EXPIRED: 'Expired',
         }
-        out['amount'] = format_satoshis(out.get('amount')) + ' BTC'
+        out['amount (BTC)'] = format_satoshis(out.get('amount'))
         out['status'] = pr_str[out.get('status', PR_UNKNOWN)]
         return out
 
@@ -577,12 +577,15 @@ class Commands:
         expiration = int(expiration)
         req = self.wallet.make_payment_request(addr, amount, memo, expiration)
         self.wallet.add_payment_request(req, self.config)
-        return self._format_request(req)
+        out = self.wallet.get_payment_request(addr, self.config)
+        return self._format_request(out)
 
     @command('wp')
     def signrequest(self, address):
         "Sign payment request with an OpenAlias"
         alias = self.config.get('alias')
+        if not alias:
+            raise BaseException('No alias in your configuration')
         alias_addr = self.contacts.resolve(alias)['address']
         self.wallet.sign_payment_request(address, alias, alias_addr, self.password)
 
@@ -613,13 +616,11 @@ param_descriptions = {
     'encrypted': 'Encrypted message',
     'amount': 'Amount to be sent (in BTC). Type \'!\' to send the maximum available.',
     'requested_amount': 'Requested amount (in BTC).',
-    'csv_file': 'CSV file of recipient, amount',
+    'outputs': 'list of ["address", amount]',
 }
 
 command_options = {
-    'broadcast':   (None, "--broadcast",   "Broadcast the transaction to the Bitcoin network"),
     'password':    ("-W", "--password",    "Password"),
-    'concealed':   ("-C", "--concealed",   "Don't echo seed to console when restoring"),
     'receiving':   (None, "--receiving",   "Show only receiving addresses"),
     'change':      (None, "--change",      "Show only change addresses"),
     'frozen':      (None, "--frozen",      "Show only frozen addresses"),
@@ -635,8 +636,6 @@ command_options = {
     'entropy':     (None, "--entropy",     "Custom entropy"),
     'language':    ("-L", "--lang",        "Default language for wordlist"),
     'gap_limit':   ("-G", "--gap",         "Gap limit"),
-    'mpk':         (None, "--mpk",         "Restore from master public key"),
-    'deserialized':("-d", "--deserialized","Return deserialized transaction"),
     'privkey':     (None, "--privkey",     "Private key. Set to '?' to get a prompt."),
     'unsigned':    ("-u", "--unsigned",    "Do not sign transaction"),
     'domain':      ("-D", "--domain",      "List of addresses"),
@@ -657,8 +656,8 @@ arg_types = {
     'pubkeys': json.loads,
     'inputs': json.loads,
     'outputs': json.loads,
-    'tx_fee': lambda x: Decimal(x) if x is not None else None,
-    'amount': lambda x: Decimal(x) if x!='!' else '!',
+    'tx_fee': lambda x: float(x) if x is not None else None,
+    'amount': lambda x: float(x) if x!='!' else '!',
 }
 
 config_variables = {
@@ -707,7 +706,7 @@ def add_network_options(parser):
 from util import profiler
 
 @profiler
-def get_parser(run_gui, run_daemon, run_cmdline):
+def get_parser():
     # parent parser, because set_default_subparser removes global options
     parent_parser = argparse.ArgumentParser('parent', add_help=False)
     group = parent_parser.add_argument_group('global options')
@@ -723,21 +722,21 @@ def get_parser(run_gui, run_daemon, run_cmdline):
     # gui
     parser_gui = subparsers.add_parser('gui', parents=[parent_parser], description="Run Electrum's Graphical User Interface.", help="Run GUI (default)")
     parser_gui.add_argument("url", nargs='?', default=None, help="bitcoin URI (or bip70 file)")
-    parser_gui.set_defaults(func=run_gui)
-    parser_gui.add_argument("-g", "--gui", dest="gui", help="select graphical user interface", choices=['qt', 'lite', 'gtk', 'text', 'stdio', 'jsonrpc'])
+    #parser_gui.set_defaults(func=run_gui)
+    parser_gui.add_argument("-g", "--gui", dest="gui", help="select graphical user interface", choices=['qt', 'lite', 'gtk', 'kivy', 'text', 'stdio', 'jsonrpc'])
     parser_gui.add_argument("-m", action="store_true", dest="hide_gui", default=False, help="hide GUI on startup")
     parser_gui.add_argument("-L", "--lang", dest="language", default=None, help="default language used in GUI")
     add_network_options(parser_gui)
     # daemon
     parser_daemon = subparsers.add_parser('daemon', parents=[parent_parser], help="Run Daemon")
     parser_daemon.add_argument("subcommand", choices=['start', 'status', 'stop'])
-    parser_daemon.set_defaults(func=run_daemon)
+    #parser_daemon.set_defaults(func=run_daemon)
     add_network_options(parser_daemon)
     # commands
     for cmdname in sorted(known_commands.keys()):
         cmd = known_commands[cmdname]
         p = subparsers.add_parser(cmdname, parents=[parent_parser], help=cmd.help, description=cmd.description)
-        p.set_defaults(func=run_cmdline)
+        #p.set_defaults(func=run_cmdline)
         if cmd.requires_password:
             p.add_argument("-W", "--password", dest="password", default=None, help="password")
         for optname, default in zip(cmd.options, cmd.defaults):
