@@ -21,10 +21,10 @@ from threading import Lock
 
 from bitcoin import Hash, hash_encode
 from transaction import Transaction
-from util import print_error, print_msg
+from util import print_error, print_msg, ThreadJob
 
 
-class WalletSynchronizer():
+class Synchronizer(ThreadJob):
     '''The synchronizer keeps the wallet up-to-date with its set of
     addresses and their transactions.  It subscribes over the network
     to wallet addresses, gets the wallet to generate new addresses
@@ -46,12 +46,6 @@ class WalletSynchronizer():
         self.lock = Lock()
         self.initialize()
 
-    def print_error(self, *msg):
-        print_error("[Synchronizer]", *msg)
-
-    def print_msg(self, *msg):
-        print_msg("[Synchronizer]", *msg)
-
     def parse_response(self, response):
         if response.get('error'):
             self.print_error("response error:", response)
@@ -62,6 +56,9 @@ class WalletSynchronizer():
         return (not self.requested_tx and not self.requested_histories
                 and not self.requested_addrs)
 
+    def release(self):
+        self.network.unsubscribe(self.addr_subscription_response)
+
     def add(self, address):
         '''This can be called from the proxy or GUI threads.'''
         with self.lock:
@@ -79,14 +76,15 @@ class WalletSynchronizer():
         if not params:
             return
         addr = params[0]
-        if addr in self.requested_addrs:  # Notifications won't be in
-            self.requested_addrs.remove(addr)
         history = self.wallet.get_address_history(addr)
         if self.wallet.get_status(history) != result:
             if self.requested_histories.get(addr) is None:
+                self.requested_histories[addr] = result
                 self.network.send([('blockchain.address.get_history', [addr])],
                                   self.addr_history_response)
-                self.requested_histories[addr] = result
+        # remove addr from list only after it is added to requested_histories
+        if addr in self.requested_addrs:  # Notifications won't be in
+            self.requested_addrs.remove(addr)
 
     def addr_history_response(self, response):
         params, result = self.parse_response(response)
@@ -94,25 +92,25 @@ class WalletSynchronizer():
             return
         addr = params[0]
         self.print_error("receiving history", addr, len(result))
-        server_status = self.requested_histories.pop(addr)
-
-        # Check that txids are unique
+        server_status = self.requested_histories[addr]
         hashes = set(map(lambda item: item['tx_hash'], result))
+        hist = map(lambda item: (item['tx_hash'], item['height']), result)
+        # Note if the server hasn't been patched to sort the items properly
+        if hist != sorted(hist, key=lambda x:x[1]):
+            self.network.interface.print_error("serving improperly sorted address histories")
+        # Check that txids are unique
         if len(hashes) != len(result):
             self.print_error("error: server history has non-unique txids: %s"% addr)
-            return
-
         # Check that the status corresponds to what was announced
-        hist = map(lambda item: (item['tx_hash'], item['height']), result)
-        if self.wallet.get_status(hist) != server_status:
+        elif self.wallet.get_status(hist) != server_status:
             self.print_error("error: status mismatch: %s" % addr)
-            return
-
-        # Store received history
-        self.wallet.receive_history_callback(addr, hist)
-
-        # Request transactions we don't have
-        self.request_missing_txs(hist)
+        else:
+            # Store received history
+            self.wallet.receive_history_callback(addr, hist)
+            # Request transactions we don't have
+            self.request_missing_txs(hist)
+        # Remove request; this allows up_to_date to be True
+        self.requested_histories.pop(addr)
 
     def tx_response(self, response):
         params, result = self.parse_response(response)
@@ -128,9 +126,10 @@ class WalletSynchronizer():
             return
         self.wallet.receive_tx_callback(tx_hash, tx, tx_height)
         self.requested_tx.remove((tx_hash, tx_height))
-        self.print_error("received tx:", tx_hash, len(tx.raw))
+        self.print_error("received tx %s height: %d bytes: %d" %
+                         (tx_hash, tx_height, len(tx.raw)))
         # callbacks
-        self.network.trigger_callback('new_transaction', (tx,))
+        self.network.trigger_callback('new_transaction', tx)
         if not self.requested_tx:
             self.network.trigger_callback('updated')
 
@@ -164,7 +163,7 @@ class WalletSynchronizer():
             self.print_error("missing tx", self.requested_tx)
         self.subscribe_to_addresses(set(self.wallet.addresses(True)))
 
-    def main_loop(self):
+    def run(self):
         '''Called from the network proxy thread main loop.'''
         # 1. Create new addresses
         self.wallet.synchronize()
